@@ -25,18 +25,20 @@ sub _render_json {
     my $output = $_JSON->encode($stuff);
     $output =~ s!([+<>])!$_ESCAPE{$1}!g;
 
+    # defense from JSON hijacking
+    if ((!$c->request->header('X-Requested-With')) && ($c->req->user_agent||'') =~ /android/i && defined $c->req->header('Cookie') && ($c->req->method||'GET') eq 'GET') {
+        my $res = $c->create_response(403);
+        $res->content_type('text/html; charset=utf-8');
+        $res->content("Your request is maybe JSON hijacking.\nIf you are not a attacker, please add 'X-Requested-With' header to each request.");
+        $res->content_length(length $res->content);
+        return $res;
+    }
+
     my $res = $c->create_response(200);
 
     my $encoding = $c->encoding();
     $encoding = lc($encoding->mime_name) if ref $encoding;
-    # chrome bug
-    if ( ( $c->req->user_agent || '' ) =~ /Chrome/ and
-        ( $c->req->env->{'HTTP_X_REQUESTED_WITH'} || '' ) ne 'XMLHttpRequest' ) {
-        $res->content_type("text/html; charset=$encoding");
-    }
-    else {
-        $res->content_type("application/json; charset=$encoding");
-    }
+    $res->content_type("application/json; charset=$encoding");
 
     # add UTF-8 BOM if the client is Safari
     if ( ( $c->req->user_agent || '' ) =~ m/Safari/ and $encoding eq 'utf-8' ) {
@@ -98,7 +100,41 @@ You can use JSONP by using L<Plack::Middleware::JSONP>.
 
 =head1 JSON and security
 
-See the L<hasegawayosuke's article(Japanese)|http://www.atmarkit.co.jp/fcoding/articles/webapp/05/webapp05a.html>.
+=over 4
+
+=item Browse the JSON files directly.
+
+This module escapes '<', '>', and '+' characters by "\uXXXX" form. Browser don't detects the JSON as HTML.
+
+And also this module outputs "X-Content-Type-Options: nosniff" header for IEs.
+
+It's good enough, I hope.
+
+=item JSON Hijacking
+
+Latest browsers doesn't have a JSON hijacking issue(I hope). __defineSetter__ or UTF-7 attack was resolved by browsers.
+
+But Firefox<=3.0.x and Android phones have issue on Array constructor, see L<http://d.hatena.ne.jp/ockeghem/20110907/p1>.
+
+Firefox<=3.0.x was outdated. Web application developers doesn't need to add workaround for it, see L<http://en.wikipedia.org/wiki/Firefox#Version_release_table>.
+
+L<Amon2::Plugin::Web::JSON> have a JSON hijacking detection feature. Amon2::Plugin::Web::JSON returns "403 Forbidden" response if following pattern request.
+
+=over 4
+
+=item The request have 'Cookie' header.
+
+=item The request doesn't have 'X-Requested-With' header.
+
+=item The request contains /android/i string in 'User-Agent' header.
+
+=item Request method is 'GET'
+
+=back
+
+=back
+
+See also the L<hasegawayosuke's article(Japanese)|http://www.atmarkit.co.jp/fcoding/articles/webapp/05/webapp05a.html>.
 
 =head1 THANKS TO
 
